@@ -27,8 +27,6 @@
 
 #include "src/cpu.h"
 #include "src/looprestoration.h"
-
-#include "common/attributes.h"
 #include "src/tables.h"
 
 #if BITDEPTH == 8
@@ -258,7 +256,7 @@ static void sgr_filter_neon(pixel *const dst, const ptrdiff_t dst_stride,
 #endif // ARCH_AARCH64
 #endif // BITDEPTH == 8
 
-void bitfn(dav1d_loop_restoration_dsp_init_arm)(Dav1dLoopRestorationDSPContext *const c) {
+COLD void bitfn(dav1d_loop_restoration_dsp_init_arm)(Dav1dLoopRestorationDSPContext *const c) {
     const unsigned flags = dav1d_get_cpu_flags();
 
     if (!(flags & DAV1D_ARM_CPU_FLAG_NEON)) return;
