@@ -35,7 +35,7 @@ typedef struct DemuxerContext DemuxerContext;
 void init_demuxers(void);
 int input_open(DemuxerContext **const c_out,
                const char *const name, const char *const filename,
-               unsigned fps[2], unsigned *num_frames);
+               unsigned fps[2], unsigned *num_frames, unsigned timebase[2]);
 int input_read(DemuxerContext *ctx, Dav1dData *data);
 void input_close(DemuxerContext *ctx);
 
